/*
 * $Id: $
 */

/*
 * Harbour Project source code:
 * Native compiler object module generation from Harbour C output.
 *
 * Copyright 2001 Jos Laln <dezac@corevia.com>
 * www - http://www.harbour-project.org
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA (or visit
 * their web site at http://www.gnu.org/).
 *
 */

#include "hbcomp.h"

#if defined(__BORLANDC__)
   #include <dir.h>
#endif

/* TODO: Allocate buffer dinamically */
#define HB_CFG_LINE_LEN   100

/* Builds platform dependant object module from Harbour C output */
void hb_compGenCObj( PHB_FNAME pFileName )
{
   char szFileName[ _POSIX_PATH_MAX ];
   char szLine[ HB_CFG_LINE_LEN ];
   char szCommandLine[ HB_CFG_LINE_LEN * 2 ];
   char * pszCompiler[ HB_CFG_LINE_LEN ];
   char * pszOptions[ HB_CFG_LINE_LEN ];
   char * szEnv;
   FILE * yyc;
   char * szCfg;
   BOOL bVerbose = FALSE;   /* Don't show C compiler messages (default). */

   /* First pass: build the C output */

   /* Force file extension to avoid collisions when called from a make utility */
   pFileName->szExtension = ".c";
   hb_fsFNameMerge( szFileName, pFileName );
   hb_compGenCCode( hb_comp_pFileName );

   /* Begin second pass */

   /* TODO: Check $(PATH) in other OS's */
#if defined(__MSDOS__) || defined(__WIN32__) || defined(_Windows)
   szCfg = searchpath( "harbour.cfg" );
#endif

   /* If "harbour.cfg" doesn't exists along the path,
      try to open in current directory.
   */
   yyc = fopen( szCfg?szCfg:"harbour.cfg", "rt" );
   if( ! yyc )
   {
#if 0
      /* QUESTION: Add a new error to Harbour ?
      hb_compGenError( hb_comp_szErrors, 'E', HB_COMP_ERR_OPEN_CFG, szFileName, NULL );
      */
#else
      printf( "\nCan't find harbour.cfg file.\n" );
      return;
#endif
   }

   while( fgets( szLine, HB_CFG_LINE_LEN, yyc ) != NULL )
   {
      ULONG ulLen;
      char * szStr = szLine;
      char * szToken;

      /* Trim left */
      while( HB_ISSPACE( *szStr ) )
         szStr++;

      /* Trim right */
      ulLen = strlen( szStr );
      while( ulLen && HB_ISSPACE( szStr[ ulLen - 1 ] ) )
         ulLen--;

      szStr[ ulLen ] = '\0';
      /* TODO: Check for comments within macros, i.e: CC=bcc32 #comment */

      if( szStr )
      {
         szToken = strtok( szStr, "=" );

         if( szToken )
         {
            /* Checks compiler name */
            if( ! hb_stricmp( szToken, "CC" ) )
            {
               szToken = strtok( NULL, "=" );
               if( szToken ) /* If empty, preserve last value */
                  sprintf( pszCompiler, "%s", szToken );
            }

            /* Checks optional switches */
            if( szToken && ! hb_stricmp( szToken, "CFLAGS" ) )
            {
               szToken = strtok( NULL, "=" );
               if( szToken )
                  sprintf( pszOptions, "%s", szToken );
            }

            /* Wanna see C compiler output ? */
            if( szToken && ! hb_stricmp( szToken, "VERBOSE" ) )
            {
               szToken = strtok( NULL, "=" );
               if( szToken && ! hb_stricmp( szToken, "YES" ) )
                  bVerbose = TRUE;
            }
         }
      }
   }

   fclose( yyc );

   if( ! hb_comp_bQuiet )
   {
      printf( "Building object module output for \'%s\'...", szFileName );
      fflush( stdout );
   }

   if( pszCompiler )
   {
      if( bVerbose )
      {
         printf( "\n" ) ;
         sprintf( szCommandLine, "%s %s %s", pszCompiler, pszOptions, szFileName );
         printf( "\n" ) ;
      }
      else
      {
#if defined(__MSDOS__) || defined(__WIN32__) || defined(_Windows)
         sprintf( szCommandLine, "%s %s %s > nul", pszCompiler, pszOptions, szFileName );
#elif defined( OS_UNIX_COMPATIBLE )
         sprintf( szCommandLine, "%s %s %s", pszCompiler, pszOptions, szFileName );
#endif
      }

      /* QUESTION: Check if compiler returned ENOxxx errors ? */
      /* Compile it! */
      system( szCommandLine );

      /* TOCHECK: Is unlink ANSI C ? */
      /* QUESTION: Add an option to not delete this file ? */
      /* Delete intermediate .c file */
      unlink( ( char * ) szFileName );
   }

   if( ! hb_comp_bQuiet )
      printf( "Done.\n" );
}
